/*
   Copyright 2009 Andrea Pagnani and Alfredo Braunstein

   This file is part of DILPERC (DILuted discrete PERCeptron analysis tool).

   DILPERC is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License Version 2 
   as published by the Free Software Foundation 

   DILPERC is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with DILPERC; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include <stdlib.h>

#include "random.h"
#include "bp.h"


int converge(int T)
{
	int t;
	double diff = 0;
	for (t = 0; t < T; ++t) {
		diff = iterate();
		printf("err: %i %g\n", t, diff);
		if (diff < err)
			return 1;
	}
	return 0;
}


int main(int argc, char ** argv) 
{
	char * inputfile = NULL;
	char c;
	int T = 100;
	err = 1e-5;
	usrand(123);
	while((c=getopt(argc, argv, "ln:m:q:t:f:h:c:e:s:d:b:")) != -1 || argc ==1) {
		switch(c) {
		case 'l':
			nonlinear = 0;
			break;
		case 'r':
			err = atof(optarg);
			break;
		case 'c':
			deltahdil = atof(optarg);
			break;
		case 'b':
			deltahdil = atof(optarg);
			break;
		case 'e':
			nmax = atoi(optarg);
			break;
		case 'd':
			deltan = atoi(optarg);
			break;
		case 'h':
			hdil = atof(optarg);
			break;
		case 'n':
			N = atoi(optarg);
			break;
		case 's':
			usrand((unsigned)atoi(optarg));
			break;
		case 'f':
			inputfile = optarg;
			break;
		case 'm':
			M = atoi(optarg);
			break;
		case 'q':
			Q = atoi(optarg);
			break;
		case 't':
			T = atoi(optarg);
			break;
		default:
			fprintf(stderr, "Usage: %s [options]\n"
				"\t-n\t\t <number of inputs>\n" 
				"\t-m\t\t <number of patterns>\n"
				"\t-q\t\t <synaptic depth>\n" 
				"\t-h\t\t <external dilution field h>\n"
				"\t-c\t\t <delta h (!= 0 paracooling) otherwise simple convergence>\n" 
				"\t-d\t\t <delta n (!= 0 decimation) otherwise simple convergence>\n" 
				"\t-e\t\t <nmax for decimation (default=N) >\n" 
				"\t-t\t\t <maximum number of iterations>\n" 
				"\t-f\t\t <nomefile dataset>\n"
				"\t-s\t\t <random seed>\n"
				
				"\t-h\t\t <this help>\n",
				argv[0]);
			return (-1);
		}
	}
	if (deltahdil && deltan) {
		fprintf( stderr, "either decimation or paracooling\n");
		exit(1);
	}
	
	init();
	read_matrix(inputfile);

	if (deltahdil)
		paracooling(T, deltahdil);
	else if (deltan)
		decimate(T, deltan, nmax);
	else
		converge(T);
	compute_marginals();
	print_marginals();
	
	printf("S: %f\n", entropy());

	return 0;
}

